@@ -489,10 +489,19 @@ imxGetExpectationComponent <- function(model, component, defvar.row=1, subname=m
 	if(is.null(model[[subname]]$expectation) && (class(model[[subname]]$fitfunction) %in% "MxFitFunctionMultigroup") ){
 		submNames <- sapply(strsplit(model$fitfunction$groups, ".", fixed=TRUE), "[", 1)
 		got <- list()
-		for(amod in submNames){
-			got[[amod]] <- imxGetExpectationComponent(model, component, defvar.row=1, subname=amod)
+		if(length(defvar.row) == length(submNames) && !is.null(names(defvar.row)) && all(names(defvar.row) %in% submNames) && !any(duplicated(names(defvar.row)))){
+			theDefVarRows <- defvar.row
+		} else if(length(defvar.row) == 1) {
+			theDefVarRows <- rep(defvar.row[1], length(submNames))
+			names(theDefVarRows) <- submNames
+		} else {
+			stop("Terribly sorry, but I don't understand your defvar.row argument.\nIt should be a single numeric value, or a named vector that corresponds to each submodel.")
 		}
-		if(component=='vector' || tolower(component)=='standvector'){got <- unlist(got)}
+		for(amod in submNames){
+			got[[amod]] <- imxGetExpectationComponent(model, component,
+				defvar.row=theDefVarRows[[amod]], subname=amod)
+		}
+		if(length(component) == 1 && (component=='vector' || tolower(component)=='standvector')){got <- unlist(got)}
 		got
 	} else if (length(component) == 1 && component == 'vector') {
 		genericGetExpectedVector(model[[subname]]$expectation, model, defvar.row, subname)
@@ -549,40 +558,108 @@ omxManifestModelByParameterJacobian <- function(model, defvar.row=1, standardize
 
 	return(jac)
 }
+minimumObservations <- function(model){
+	namespace <- imxGenerateNamespace(model)
+	flatModel <- imxFlattenModel(model, namespace, TRUE)
+	datalist <- Filter(function(x) !is(x,"MxDataDynamic"), flatModel@datasets)
+	convertArguments <- imxCheckVariables(flatModel, namespace)
+	flatModel <- constraintsToAlgebras(flatModel)
+	flatModel <- eliminateObjectiveFunctions(flatModel)
+	flatModel <- convertAlgebras(flatModel, convertArguments)
+	defVars <- generateDefinitionList(flatModel, list())
+	defVarsData <- sapply(defVars, '[', 1) + 1 # indices of the data for each defvar
+	defVarsCols <- sapply(defVars, '[', 2) + 1 # column indices of the definition variables
+	# Transfer NAMES of submodels as the names of the uniDef to mxCheckID
+	# Then use these as args to defvar.row=c(MZ=1, DZ=3) etc.
+	# Model names of corresponding definition variables
+	smartSel <- function(x){if(length(x) > 0) x[[1]] else NA}
+	defVarsMods <- sapply(strsplit(names(defVarsData), split=imxSeparatorChar, fixed=TRUE), smartSel)
+	# For each unique defVarsData,
+	defVarsDups <- duplicated(defVarsData)
+	uniData <- defVarsData[!defVarsDups]
+	uniMods <- defVarsMods[!defVarsDups]
+	uniDef <- list()
+	for(ind in 1L:length(uniData)){
+		tmpData <- datalist[[ind]]$observed
+		curDefVarCols <- defVarsCols[defVarsData %in% ind]
+		tmpDef <- tmpData[, curDefVarCols, drop=FALSE]
+		uniDef[[ind]] <- which(!duplicated(tmpDef)) # unique(tmpDef, MARGIN=1)
+	}
+	names(uniDef) <- uniMods
+	#  grab tmpData <- datalist[[aUniDefVarData]]$observed
+	#  grab ALL the columns that go with those data:
+	#    s <- defVarsData[defVarsData %in% aUniDefVarData]
+	#    curDefVarCols <- defVarsCols[s]
+	#    tmpDef <- tmpData[,curDefVarCols]
+	#  Find unique patterns of defvar values with unique(tmpDef, margin=??)
+	#return(min(c(as.numeric(dataObservations), as.numeric(fitfunctionObservations)), na.rm=TRUE))
+	return(uniDef)
+}
 
-mxCheckIdentification <- function(model, details=TRUE){
-  warnModelCreatedByOldVersion(model)
+
+mxCheckIdentification <- function(model, details=TRUE, nrows=2, exhaustive=FALSE, silent=FALSE){
+	warnModelCreatedByOldVersion(model)
 	notAllowedFits <- c("MxFitFunctionAlgebra", "MxFitFunctionRow", "MxFitFunctionR")
 	if( class(model$fitfunction) %in% notAllowedFits ){
 		msg <- paste("Identification check is not possible for models with", omxQuotes(notAllowedFits), 'fit functions.\n', "If you have a multigroup model, use mxFitFunctionMultigroup.")
 		stop(msg, call.=FALSE)
 	}
-	if(imxHasDefinitionVariable(model)){
-		stop("Beep beep ribby ribby.  I found definition variables in your model.\nI might not give you the identification answer you're looking for in this case.  See ?mxCheckIdentification.")
-	}
 	eps <- 1e-17
 	theParams <- omxGetParameters(model)
-	jac <- omxManifestModelByParameterJacobian(model)
 	if(imxHasConstraint(model)){
     # Better if there was a separate compute step to estimate the constraintJacobian
 		tmpModel <- mxModel(model, mxComputeSequence(list(mxComputeNumericDeriv(hessian=FALSE), mxComputeReportDeriv())))
 		tmpModel <- mxRun(tmpModel, silent=TRUE)
 		cjac <- tmpModel$output$constraintJacobian
 		# drop model name from constraint name
-		rownames(cjac) <- sapply(strsplit(rownames(cjac), fixed=TRUE, split=imxSeparatorChar), '[', 2)
+		if(length(rownames(cjac))){
+			rownames(cjac) <- sapply(strsplit(rownames(cjac), fixed=TRUE, split=imxSeparatorChar), '[', 2)
+		}
 	} else {
 		cjac <- matrix(, nrow=0, ncol=length(theParams))
 		colnames(cjac) <- names(theParams)
 	}
+	if(imxHasDefinitionVariable(model)){
+		uniRow <- minimumObservations(model)
+     # OR just cycle through combinations of def vars
+     # 2 group ex with g1 has 1 5 9 13 unique rows
+     #                 g2 has 1 2 4 unique rows
+     # g1=1, g2=1
+     # g1=5, g2=2
+     # g1=9, g2=4
+     # g1=13,g2=1
+		uniMat <- suppressWarnings(do.call(cbind, uniRow))
+		minRow <- 1
+		maxRow <- nrow(uniMat)
+		jac <- cjac
+		arow <- 1
+		jdim <- 0
+		jacL <- list()
+		while(arow <= min(maxRow, nrows) && (exhaustive || jdim != length(theParams))){
+			jacL[[arow]] <- omxManifestModelByParameterJacobian(model, defvar.row=uniMat[arow,]) # put in here
+			rownames(jacL[[arow]]) <- paste0(rownames(jacL[[arow]]), 'def', paste(uniMat[arow,], collapse='_'))
+			# TODO Put defvar row that corresponds to group.  If multigroup, check the beginning of strsplit(rownames(), imxSeparatorChar).  This inside omxManifestModelByParameterJacobian() function definition?
+			# paste(names(uniMat[minRow,]), uniMat[minRow,], sep='_')
+			jac <- rbind(jac, jacL[[arow]])
+			arow <- arow + 1
+			if(!exhaustive) jdim <- qr(jac)$rank # Arg.  Doing full QR every time.  Inefficient
+		}
+		if(!silent && !exhaustive && jdim != length(theParams)){
+			message(paste0('We tried ', arow-1, ' definition variable value rows.\nThe model is not yet locally identified.\nThe model might be identified if you try more definition variable rows, but it will take a while to run.\nUse mxCheckIdentification(YourModel, nrows=Inf, exhaustive=TRUE) to evaluate all possible rows,\nor set nrows to a larger number.'))
+		}
+		jac <- do.call(rbind, jacL) # rbind-ing AGAIN
+	} else {
+		jac <- omxManifestModelByParameterJacobian(model)
+	}
 	# Concatenate via rows the model and constraint Jacobians
 	jac <- rbind(jac, cjac)
 	# Check that rank of jac == length(theParams)
 	rank <- qr(jac)$rank
 	if(rank == length(theParams)){
-		message("Model is locally identified")
+		if(!silent) message("Model is locally identified")
 		stat <- TRUE
 	} else {
-		message("Model is not locally identified")
+		if(!silent) message("Model is not locally identified")
 		stat <- FALSE
 	}
 	if(details == TRUE){
@@ -655,12 +732,16 @@ generateNormalData <- function(model, nrows, subname, empirical, returnModel, us
 	# Check for definition variables
 	if(imxHasDefinitionVariable(model[[subname]])){
     if (is.null(origData)) {
-      stop("Definition variable(s) found, but no data is available")
+      stop("Definition variable(s) found, but no data are available")
     }
 		if (origData$type != 'raw') {
 			stop(paste("Definition variable(s) found, but original data is type",
 				omxQuotes(origData$type)))
 		}
+		#Trying to use .rmvnorm() with empirical=TRUE and nrow=1 leads to an R error raised in MASS::mvrnorm():
+		if(empirical){
+			stop("argument 'empirical=TRUE' to mxGenerateData() is not compatible with definition variables")
+		}
 		origData <- origData$observed
 		if(nrows != nrow(origData)){
 			stop("Definition variable(s) found, but the number of rows in the data do not match the number of rows requested for data generation.")
