@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2021 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -137,6 +137,12 @@ generateParameterList <- function(flatModel, dependencies, freeVarGroups) {
 
 	if (length(pList)) for(i in 1:length(pList)) {
 		original <- pList[[i]]
+    bounds <- unlist(original[1:2])
+    if (all(!is.na(bounds)) && diff(bounds) <= 0) {
+      stop(paste("Parameter", omxQuotes(names(pList)[i]),
+                 "has a lower bound (", bounds[1], ") greater than or",
+                 "equal to an upper bound (", bounds[2], ")"), call.=FALSE)
+    }
 		svalues <- original[5:length(original)]
 		svalue <- NA
 		if (length(svalues) > 1) {
@@ -260,7 +266,8 @@ updateModelMatrices <- function(model, flatModel, values) {
 updateModelAlgebras <- function(model, flatModel, values) {
 	aNames <- names(flatModel@algebras)
 	oNames <- names(flatModel@fitfunctions)
-	aList <- append(aNames, oNames)
+	pNames <- names(flatModel@penalties)
+	aList <- append(aNames, append(oNames, pNames))
 	if(length(aList) != length(values)) {
 		stop(paste("This model has", length(aList),
 			"algebras, but the backend has returned", length(values),
@@ -328,12 +335,15 @@ updateModelEntitiesTargetModel <- function(model, entNames, values, modelNameMap
 					if(is(candidate,"MxFitFunctionGREML")){
 						candidate@MLfit <- attr(value,"MLfit")
 						candidate@result <- as.matrix(as.numeric(value))
+            candidate@numObsAdjust <- attr(value,"numObsAdjust")
 					} else{
 						candidate@result <- as.matrix(value)
 						attr <- attributes(value)
 						attr$dim <- NULL
 						candidate@info <- attr
 					}
+				} else if (is(candidate,"MxPenalty")) {
+          candidate@result <- as.matrix(value)
 				} else if(is(candidate, "MxMatrix")) {
 					if(candidate@name=="filteredDataRow" && !candidate@.persist){next} #bit of a hack
 					if (any(dim(candidate@values) != dim(value))) {
@@ -398,19 +408,20 @@ clearModifiedSinceRunRecursive <- function(model) {
 imxLocateIndex <- function(model, name, referant) {
 	if (length(name) == 0) return(name)
 #	if (length(name) > 1) browser()
-	if (is.na(name)) { return(as.integer(name)) }
-  if (is.integer(name)) stop("imxLocateIndex called more than once, OpenMx bug")
+	if (all(is.na(name))) { return(as.integer(name)) }
+  if (any(is.integer(name))) stop("imxLocateIndex called more than once, OpenMx bug")
 	mNames <- names(model@matrices)
 	aNames <- names(model@algebras)
 	fNames <- names(model@fitfunctions)
+  pNames <- names(model@penalties)
 	eNames <- names(model@expectations)
 	dNames <- names(model@datasets)
 	matrixNumber <- match(name, mNames)
-	algebraNumber <- match(name, append(aNames, fNames))
+	algebraNumber <- match(name, append(aNames, c(fNames, pNames)))
 	dataNumber <- match(name, dNames)
 	expectationNumber <- match(name, eNames)
-	if (is.na(matrixNumber) && is.na(algebraNumber)
-		&& is.na(dataNumber) && is.na(expectationNumber)) {
+	if (all(is.na(matrixNumber)) && all(is.na(algebraNumber)) &&
+        all(is.na(dataNumber)) && all(is.na(expectationNumber))) {
 		reftype <- "named reference"
 		if (typeof(referant) == "S4") {
 			reftype <- 'object'
@@ -419,11 +430,11 @@ imxLocateIndex <- function(model, name, referant) {
 			"does not exist.  It is used by",
 			     reftype, omxQuotes(referant), ".")
 		stop(msg, call.=FALSE)
-	} else if (!is.na(matrixNumber)) {
+	} else if (all(!is.na(matrixNumber))) {
 		return(- matrixNumber)
-	} else if (!is.na(dataNumber)) {
+	} else if (all(!is.na(dataNumber))) {
 		return(dataNumber - 1L)
-	} else if (!is.na(expectationNumber)) {
+	} else if (all(!is.na(expectationNumber))) {
 		return(expectationNumber - 1L)
 	} else {
 		return(algebraNumber - 1L)
