@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2021 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -121,6 +121,9 @@ variablesArgumentRAM <- function(model, manifestVars, latentVars, productVars, s
 			checkVariables(model, latentVars, manifestVars)
 			model <- addVariablesRAM(model, latentVars, manifestVars)
 			model[['expectation']]$isProductNode <- colnames(model$A) %in% productVars
+      if (length(productVars) && !is.null(model[['M']])) {
+        model[['M']]$values[1,productVars] <- 1
+      }
 		}
 		if (length(submodels)) for(i in 1:length(submodels)) {
 			model <- addSingleNamedEntity(model, submodels[[i]])
@@ -159,7 +162,7 @@ removeVariablesRAM <- function(model, latent, manifest) {
 	model[['A']] <- A
 	model[['S']] <- S
 	Thresh <- model[['Thresholds']]
-	if(!all.na(Thresh)) {
+	if(!isAllNa(Thresh)) {
 		newCols <- setdiff(colnames(Thresh), manifest)
 		newRows <- nrow(Thresh) - min(colSums(is.na(Thresh@values[,newCols])))
 		model[['Thresholds']] <- Thresh[1:newRows,newCols]
@@ -406,7 +409,7 @@ insertAllThresholdsRAM <- function(model, thresholds) {
 		thisThresh <- thresholds[[i]]
 		values <- thisThresh@values
 		msg = NULL
-		if(all.na(values)) {
+		if(isAllNa(values)) {
 			msg = paste("The thresholds you are attempting to specify",
 				"does not have any starting values, but type='RAM' models require them.")
 		}
@@ -511,8 +514,12 @@ insertPathRAM <- function(path, model) {
 	S_labels <- S@labels
 	S_lbound <- S@lbound
 	S_ubound <- S@ubound
+  selVec <- model[['selectionVector']]
+  selPlan <- model$expectation$selectionPlan
 
 	legalVars <- c(colnames(A), "one")
+	isProductNode <- model$expectation$isProductNode
+	names(isProductNode) <- colnames(A)
 
 	for(i in 0:(maxlength - 1)) {
 		from <- allfrom[[i %% length(allfrom) + 1]]
@@ -524,6 +531,7 @@ insertPathRAM <- function(path, model) {
 		nextubound <- allubound[[i %% length(allubound) + 1]]
 		nextlbound <- alllbound[[i %% length(alllbound) + 1]]
 		nextjoinKey <- alljoinKey[[i %% length(alljoinKey) + 1]]
+    nextstep <- path@step[[i %% length(path@step) + 1]]
 
 		if (!is.na(nextjoinKey)) {
 			upperFrom <- strsplit(from, imxSeparatorChar, fixed = TRUE)
@@ -635,6 +643,10 @@ insertPathRAM <- function(path, model) {
 				stop(paste('The means path must be a single-headed arrow\n',
 					   'path from "one" to', omxQuotes(to)), call. = FALSE)
 			}
+		  if (isProductNode[to]) {
+		    stop(paste('Cannot change mean of product node', omxQuotes(to),
+		               'from the identity value of 1'), call.=FALSE)
+		  }
 			M@free[1, to] <- nextfree
 			M@values[1, to] <- nextvalue
 			M@labels[1, to] <- nextlabel
@@ -643,7 +655,25 @@ insertPathRAM <- function(path, model) {
 			next
 		}
 
-		if (arrows == 1) {
+    if (arrows == 0) {
+      # directly modify the unfiltered covariance matrix
+      # expectation: data.frame w/ step, rowname, colname ; name of mxMatrix
+      # mxMatrix: vector of parameters to plop in
+      pair <- c(from,to)[order(match(c(from,to), legalVars))]  # canonical order
+      r1 <- data.frame(step=nextstep, from=pair[1], to=pair[2])
+      selPlan <- rbind(selPlan, r1)
+      oldSelVec <- selVec
+      selVec <- mxMatrix('Full', nrow(selPlan), 1)
+      if (!is.null(oldSelVec)) selVec[1:nrow(oldSelVec),1] <- oldSelVec
+      selVec[nrow(selVec),1]$free <- nextfree
+      selVec[nrow(selVec),1]$values <- nextvalue
+      selVec[nrow(selVec),1]$labels <- nextlabel
+      selVec[nrow(selVec),1]$ubound <- nextubound
+      selVec[nrow(selVec),1]$lbound <- nextlbound
+      p1 <- order(selPlan$step, selPlan$from, selPlan$to)
+      selPlan <- selPlan[p1,]
+      selVec <- selVec[p1,]
+    } else if (arrows == 1) {
 			A_free[to, from] <- nextfree
 			A_values[to, from] <- nextvalue
 			A_labels[to, from] <- nextlabel
@@ -692,6 +722,11 @@ insertPathRAM <- function(path, model) {
 	model[['A']] <- A
 	model[['S']] <- S
 	if (!is.null(M)) model[['M']] <- M
+  if (!is.null(selVec)) {
+    model$expectation$selectionVector <- 'selectionVector'
+    model[['selectionVector']] <- selVec
+  }
+  if (!is.null(selPlan)) model$expectation$selectionPlan <- selPlan
 	model
 }
 
@@ -755,7 +790,7 @@ createMatrixM <- function(model) {
 	variables <- c(model@manifestVars, model@latentVars)
 	len <- length(variables)
 	names <- list(NULL, variables)
-	values <- matrix(0, 1, len)
+	values <- matrix(as.numeric(model$expectation$isProductNode), 1, len)
 	labels <- matrix(as.character(NA), 1, len)
 	free <- matrix(c(rep.int(FALSE, length(model@manifestVars)),
 		rep.int(FALSE, length(model@latentVars))), 1, len)
@@ -793,25 +828,23 @@ createMatrixF <- function(model) {
 	len <- length(variables)
 	values <- diag(nrow = length(model@manifestVars), ncol = len)
 	names <- list(model@manifestVars, variables)
-	if (!is.null(model@data) && (model@data@type != 'raw' && model@data@type != 'acov')) {
-		manifestNames <- rownames(model@data@observed)
-		extraData <- setdiff(manifestNames, model@manifestVars)
+	if (!is.null(model@data)) {
+		manifestNames <- observedDataNames(model@data)
 		extraVars <- setdiff(model@manifestVars, manifestNames)
-		if (length(extraData) > 0) {
-			msg <- paste("The observed data contains the variables:",
-				omxQuotes(extraData), "that have not been declared in the",
-				"manifest variables.")
-			stop(msg, call. = FALSE)
-		}
+    # would need to exclude freq, weight, keys, etc TODO
+		## extraData <- setdiff(manifestNames, model@manifestVars)
+		## if (length(extraData) > 0) {
+		## 	msg <- paste("The observed data contains the variables:",
+		## 		omxQuotes(extraData), "that have not been declared in the",
+		## 		"manifest variables.")
+		## 	stop(msg, call. = FALSE)
+		## }
 		if (length(extraVars) > 0) {
 			msg <- paste("The manifest variables include",
 				omxQuotes(extraVars), "that have not been found in the",
 				"observed data.")
 			stop(msg, call. = FALSE)
 		}
-		dimnames(values) <- names
-		values <- values[manifestNames,]
-		names <- list(manifestNames, variables)
 	}
 	free <- matrix(FALSE, length(model@manifestVars), len)
 	labels <- matrix(as.character(NA), length(model@manifestVars), len)
@@ -871,16 +904,16 @@ addVariablesMatrix <- function(oldmatrix, value, model, newLatent, newManifest)
 	currentLatent <- length(model@latentVars) - newLatent
 	newSize <- length(model@manifestVars) + length(model@latentVars)
 	if (currentManifest > 0) {
-		manifestXmanifest <- oldmatrix[1 : currentManifest, 1 : currentManifest]
+		manifestXmanifest <- oldmatrix[1 : currentManifest, 1 : currentManifest, drop=FALSE]
 	} else {
 		manifestXmanifest <- matrix(value, 0, 0)
 	}
 	if (currentLatent > 0) {
 		latentStart <- currentManifest + 1
 		latentEnd <- currentManifest + currentLatent
-		manifestXlatent <- oldmatrix[1 : currentManifest, latentStart : latentEnd]
-		latentXmanifest <- oldmatrix[latentStart : latentEnd, 1 : currentManifest]
-		latentXlatent <- oldmatrix[latentStart : latentEnd, latentStart : latentEnd]
+		manifestXlatent <- oldmatrix[1 : currentManifest, latentStart : latentEnd, drop=FALSE]
+		latentXmanifest <- oldmatrix[latentStart : latentEnd, 1 : currentManifest, drop=FALSE]
+		latentXlatent <- oldmatrix[latentStart : latentEnd, latentStart : latentEnd, drop=FALSE]
 	} else {
 		manifestXlatent <- matrix(value, 0, 0)
 		latentXmanifest <- matrix(value, 0, 0)
