@@ -276,9 +276,6 @@ STARTSM <- mxModel("STARTS",
 startsModel <- mxRun(STARTSM)
 omxCheckCloseEnough(startsModel$output$fit, 2718.410, .15)
 
-if (.Platform$OS.type != 'windows' && parallel::detectCores() > 1) {
-	omxCheckTrue(startsModel$compute$steps[['GD']]$output$maxThreads > 1)
-}
 
 if (mxOption(key="default optimizer") == 'SLSQP') {
   startsCI <- mxModel(startsModel, mxCI('stability'))
