@@ -506,6 +506,7 @@ void printSparse(Eigen::SparseMatrixBase<T> &sm) {
 	mxLogBig(buf);
 }
 
-void AddLoadDataProvider(double version, unsigned int, class LoadDataProviderBase2 *ldp);
+void AddLoadDataProvider(double version, unsigned int otherHash,
+                         std::unique_ptr<class LoadDataProviderBase2> ldp);
 
 #endif
