@@ -1,5 +1,5 @@
-#ifndef _ALGEBRAOP_H_
-#define _ALGEBRAOP_H_
+#ifndef u_ALGEBRAOP_H_
+#define u_ALGEBRAOP_H_
 
 class omxMatrix;
 typedef void (*algebra_op_t)(FitContext *fc, class omxMatrix**, int, class omxMatrix*);
