@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,8 +27,8 @@
  *
  **********************************************************/
 
-#ifndef _OMXDATA_H_
-#define _OMXDATA_H_
+#ifndef u_OMXDATA_H_
+#define u_OMXDATA_H_
 
 #include "omxDefines.h"
 #include <R_ext/Rdynload.h>
@@ -84,8 +84,8 @@ union dataPtr {
 	double *realData;
 	int *intData;
 	dataPtr() : intData(0) {};
-	dataPtr(double *_p) : realData(_p) {};
-	dataPtr(int *_p) : intData(_p) {};
+	dataPtr(double *u_p) : realData(u_p) {};
+	dataPtr(int *u_p) : intData(u_p) {};
 	void clear() { realData=0; intData=0; };
 };
 
@@ -94,30 +94,40 @@ private:
 	dataPtr ptr;
   bool owner;
   int minValue; // for count/ordinal only
+  int maxValue; // for count/ordinal only
 public:
 	const char *name;
 	ColumnDataType type;
-	std::vector<std::string> levels;       // factors only
+	std::vector<std::string> levelNames;       // factors only
 
 	const char *typeName();
-  ColumnData(const char *_name) : owner(false), minValue(1), name(_name),
-                                  type(COLUMNDATA_INVALID) {}
-  ColumnData(const char *_name, ColumnDataType _type, int *col) :
-    ptr(col), owner(true), minValue(1), name(_name), type(_type) {}
+  ColumnData(const char *u_name) : owner(false), minValue(1), maxValue(NA_INTEGER),
+                                  name(u_name), type(COLUMNDATA_INVALID) {}
+  ColumnData(const char *u_name, ColumnDataType u_type, int *col) :
+    ptr(col), owner(true), minValue(1), maxValue(NA_INTEGER),
+    name(u_name), type(u_type) {}
   ~ColumnData() { clear(); }
   void clear();
   ColumnData clone() const;
   void setMinValue(int mv) { minValue = mv; }
+  void verifyMinValue(int nrows);
   void setZeroMinValue(int rows);
   int getMinValue() const { return minValue; }
+  int getMaxValue() const { if (maxValue==NA_INTEGER) OOPS; return maxValue; }
+  int getMaxValueUnsafe() const { return maxValue; }
+  void setMaxValue(int val) { maxValue = val; }
+  void setMaxValueFromLevels() { maxValue = minValue + levelNames.size() - 1; }
+  void setMaxValueFromData(int nrows);
+  int getNumThresholds() const { if (maxValue==NA_INTEGER) OOPS; return maxValue - minValue; }
+  int getNumOutcomes() const { return 1 + getNumThresholds(); }
   int *i() { return ptr.intData; }
   double *d() { return ptr.realData; }
-  void setOwn(double *_p) { clear(); ptr.realData = _p; owner=true; }
-  void setOwn(int *_p) { clear(); ptr.intData = _p; owner=true; }
-  void setOwn(dataPtr _p) { clear(); ptr = _p; owner=true; }
-  void setBorrow(double *_p) { clear(); ptr.realData = _p; owner=false; }
-  void setBorrow(int *_p) { clear(); ptr.intData = _p; owner=false; }
-  void setBorrow(dataPtr _p) { clear(); ptr = _p; owner=false; }
+  void setOwn(double *u_p) { clear(); ptr.realData = u_p; owner=true; }
+  void setOwn(int *u_p) { clear(); ptr.intData = u_p; owner=true; }
+  void setOwn(dataPtr u_p) { clear(); ptr = u_p; owner=true; }
+  void setBorrow(double *u_p) { clear(); ptr.realData = u_p; owner=false; }
+  void setBorrow(int *u_p) { clear(); ptr.intData = u_p; owner=false; }
+  void setBorrow(dataPtr u_p) { clear(); ptr = u_p; owner=false; }
   dataPtr steal() { dataPtr ret = ptr; ptr.clear(); return ret; }
 };
 
@@ -141,11 +151,7 @@ struct WLSVarData {
 	Eigen::ArrayXd resid;
 	int contOffset;
 	int thrOffset;
-};
-
-struct cstrCmp {
-	bool operator() (const char *s1, const char *s2) const
-	{ return strcmp(s1,s2) < 0; }
+  Eigen::MatrixXd vcov;
 };
 
 typedef std::map< const char *, int, cstrCmp > ColMapType;
@@ -158,7 +164,7 @@ class obsSummaryStats {
 	int totalExoFree;
 	const char *wlsType;
 	const char *continuousType;
-	bool wantFullWeight;
+	bool wantAsymCov;
 	std::vector<int> index; // rowMult.rows() == index.size()
 	Eigen::ArrayXd rowMult;
 
@@ -170,8 +176,8 @@ class obsSummaryStats {
 	omxMatrix* covMat;
 	omxMatrix *slopeMat; // manifest by exo predictor matrix
 	omxMatrix* meansMat;
-	omxMatrix* acovMat;			// The asymptotic covariance
-	omxMatrix *fullWeight;
+	omxMatrix *asymCov;			// The asymptotic covariance
+	omxMatrix *useWeight;
 	omxMatrix* thresholdMat;
 	std::vector< omxThresholdColumn > thresholdCols; // size() == covMat.cols()
 	ColMapType colMap;
@@ -190,15 +196,16 @@ class obsSummaryStats {
 	Eigen::ArrayXXd H21;
 
  	obsSummaryStats() :
-		wlsType(0), continuousType(0), wantFullWeight(true),
+		wlsType(0), continuousType(0), wantAsymCov(true),
 		partial(false), output(false), totalWeight(0), numOrdinal(0), numContinuous(0),
 		covMat(0), slopeMat(0), meansMat(0),
-		acovMat(0), fullWeight(0), thresholdMat(0), totalThr(0) {};
+		asymCov(0), useWeight(0), thresholdMat(0), totalThr(0) {};
 	~obsSummaryStats();
 	void setDimnames(omxData *data);
 	void permute(omxData *data);
 	void log();
 	void loadExoFree(SEXP Ref);
+  int numPredictors(int vx);
 };
 
 class omxData {
@@ -210,18 +217,21 @@ class omxData {
 	int freqCol;
 	int *currentFreqColumn;
 	std::unique_ptr< obsSummaryStats > oss;
+  int numEstimatedEntries;
 	bool parallel;
 	bool noExoOptimize;
 	bool modified;
 	double minVariance;
-	bool warnNPDacov;
+	bool warnNPDuseWeight;
 	std::vector<int> algebra;
+  double fitTolerance;
+  double gradientTolerance;
 
 	void estimateObservedStats();
-	void _prepObsStats(omxState *state, const std::vector<const char *> &dc,
+	void u_prepObsStats(omxState *state, const std::vector<const char *> &dc,
 			   std::vector<int> &exoPred, const char *type,
 			  const char *continuousType, bool fullWeight);
-	bool regenObsStats(const std::vector<const char *> &dc, const char *wlsType);
+  std::string regenObsStats(const std::vector<const char *> &dc, const char *wlsType);
 	void wlsAllContinuousCumulants(omxState *state);
 	void convertToDataFrame();
 
@@ -239,14 +249,15 @@ class omxData {
 	bool isModified() { return modified; };
 	double getMinVariance() const { return minVariance; };
 	void evalAlgebras(FitContext *fc);
+  std::string getExoPredictorName(int vx, int nx);
 
 	const char *name;
 	SEXP dataObject;                                // only used for dynamic data
 	omxMatrix* dataMat;                             // do not use directly
 	omxMatrix* meansMat;				// The means, as an omxMatrixObject
 	double numObs;						// Number of observations (sum of rowWeight)
-	const char *_type;
-	const char *getType() const { return _type; };
+	const char *u_type;
+	const char *getType() const { return u_type; };
 
 	// type=="raw"
 	struct RawData {
@@ -314,6 +325,7 @@ class omxData {
 	}
 	int numRawRows();
 	double rowMultiplier(int rx);
+  double countObs(int col);
 	bool containsNAs(int col);
 	void prohibitFactor(int col);
 	void prohibitNAdefVar(int col);
@@ -326,6 +338,7 @@ class omxData {
 		if (!oss) mxThrow("No observed summary stats");
 		return *oss;
 	};
+  bool hasVcov() { return oss && oss->perVar.size(); }
 	const char *columnName(int col);
 	bool columnIsFactor(int col);
 	bool hasSummaryStats() { return dataMat != 0 || oss; }
@@ -340,6 +353,8 @@ class omxData {
 	// When TRUE, PolychoricCor uses summary data (if possible).
 	// Both should obtain the same result when no exogenous covariates.
 	bool getNoExoOptimize() const { return noExoOptimize; };
+  double getFitTolerance() const { return fitTolerance; }
+  double getGradientTolerance() const { return gradientTolerance; }
 };
 
 omxData* omxNewDataFromMxData(SEXP dataObject, const char *name);
@@ -348,7 +363,6 @@ omxData* omxDataLookupFromState(SEXP dataObject, omxState* state);	// Retrieves
 void omxFreeData(omxData* od);					// Release any held data.
 
 /* Getters 'n Setters */
-int omxDataGetNumFactorLevels(omxData *od, int col);
 double omxDoubleDataElement(omxData *od, int row, int col);
 double *omxDoubleDataColumn(omxData *od, int col);
 int omxIntDataElement(omxData *od, int row, int col);						// Returns one data object as an integer
@@ -407,8 +421,6 @@ int omxDataNumFactor(omxData *od);                    // Number of factor column
 
 /* Function wrappers that switch based on inclusion of algebras */
 
-double omxDataDF(omxData *od);
-
 inline bool omxDataColumnIsFactor(omxData *od, int col) { return od->columnIsFactor(col); }
 
 // Should not be stored long-term because freq are updated by bootstrap
@@ -431,4 +443,4 @@ void omxData::recalcRowWeights(Eigen::ArrayBase<T1> &rowMult, std::vector<int> &
 	}
 }
 
-#endif /* _OMXDATA_H_ */
+#endif /* u_OMXDATA_H_ */
