@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -26,8 +26,8 @@
 *
 **********************************************************/
 
-#ifndef _OMXEXPECTATION_H_
-#define _OMXEXPECTATION_H_
+#ifndef u_OMXEXPECTATION_H_
+#define u_OMXEXPECTATION_H_
 
 #include "omxDefines.h"
 #include <R_ext/Rdynload.h>
@@ -45,7 +45,7 @@ class omxExpectation {					// An Expectation
 	omxMatrix *thresholdsMat;
 	double *discreteSpecPtr;
   bool discreteCheckCount;
-  bool _connectedToData;
+  bool u_connectedToData;
   const Eigen::Map<Eigen::MatrixXd> getDiscreteSpec()
   { const Eigen::Map<Eigen::MatrixXd> ds(discreteSpecPtr, 2, discreteMat->cols); return ds; }
 	omxMatrix *discreteMat;
@@ -54,19 +54,19 @@ class omxExpectation {					// An Expectation
 
 	void loadThresholds();
 protected:
-  void setConnectedToData(bool _to);
-  bool getConnectedToData() const { return _connectedToData; }
+  void setConnectedToData(bool u_to);
+  bool getConnectedToData() const { return u_connectedToData; }
 
  public:
 	int numDataColumns;
-	SEXP rObj;
+	S4 rObj;
 	const char *name;   // pointer to a static string, no need to allocate or free
 	omxData* data;
 	int numOrdinal;  // number of thresholds with matrix != 0
 	/* Replication of some of the structures from Matrix */
 	unsigned short isComplete;													// Whether or not this expectation has been initialize
 	omxState* currentState;
-	bool isClone() const;
+	bool isTopState() const;
 	int expNum;  // index in omxState's vector
 
 	// omxExpectation should not need to know about free variables.
@@ -77,8 +77,8 @@ protected:
 
 	omxExpectation(omxState *state, int num) :
 		dataColumnsPtr(0), thresholdsMat(0),
-		discreteSpecPtr(0), _connectedToData(false), discreteMat(0),
-    numDataColumns(0), rObj(0), name(0), data(0), numOrdinal(0),
+		discreteSpecPtr(0), u_connectedToData(false), discreteMat(0),
+    numDataColumns(0), name(0), data(0), numOrdinal(0),
     isComplete(false), currentState(state),
 		expNum(num), freeVarGroup(0), canDuplicate(false), dynamicDataSource(false) {};
 	virtual ~omxExpectation() {};
@@ -116,11 +116,24 @@ protected:
 	}
 
 	typedef Eigen::Matrix<int, Eigen::Dynamic, 1> DataColumnIndexVector;
-	virtual const Eigen::Map<DataColumnIndexVector> getDataColumns();
+	virtual const Eigen::Map<DataColumnIndexVector> getDataColumns();  // NOT const
 	virtual const std::vector<const char *> &getDataColumnNames() const;
 	virtual void getExogenousPredictors(std::vector<int> &out) {};
 	virtual std::vector< omxThresholdColumn > &getThresholdInfo();
 	double getThreshold(int r, int c);
+	virtual int numObservedStats() {
+    mxThrow("%s::numObservedStats() is not implemented", name);
+  }
+  virtual int numManifestVars() const { return dataColumnNames.size(); }
+  virtual int numLatentVars() const { return 0; }
+};
+
+class MVNExpectation : public omxExpectation {
+  typedef omxExpectation super;
+public:
+	MVNExpectation(omxState *state, int num) : super(state, num) {};
+	virtual int numObservedStats() override;
+  virtual void populateAttr(SEXP algebra) override;
 };
 
 	void omxCompleteExpectation(omxExpectation *ox);
@@ -246,4 +259,4 @@ void normalToStdVector(omxMatrix *cov, omxMatrix *mean, omxMatrix *slope, T Eth,
 	}
 }
 
-#endif /* _OMXEXPECTATION_H_ */
+#endif /* u_OMXEXPECTATION_H_ */
