@@ -218,7 +218,7 @@ bool condOrdByRow::eval()
 				//mxPrintMat("dataMean", ss.dataMean);
 				//mxPrintMat("resid", resid);
 				iqf = resid.transpose() * iV.selfadjointView<Eigen::Lower>() * resid;
-				double tr1 = trace_prod(iV, ss.dataCov);
+				double tr1 = trace_prod_symm(iV, ss.dataCov);
 				double logDet = 2.0 * covDecomp.log_determinant();
 				double cterm = M_LN_2PI * residSize;
 				//mxLog("[%d] iqf %f tr1 %f logDet %f cterm %f", ssx, iqf, tr1, logDet, cterm);
@@ -1049,7 +1049,7 @@ void omxFIMLFitFunction::init()
 	off->canDuplicate = TRUE;
 
 	if(expectation == NULL) {
-		omxRaiseError("FIML cannot fit without model expectations.");
+		omxRaiseErrorf("FIML cannot fit without model expectations.");
 		return;
 	}
 
