@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
  *
  */
 
-#ifndef _OMXFIMLFITFUNCTION_H_
-#define _OMXFIMLFITFUNCTION_H_
+#ifndef u_OMXFIMLFITFUNCTION_H_
+#define u_OMXFIMLFITFUNCTION_H_
 
 #include "omxFitFunction.h"
 #include "omxSadmvnWrapper.h"
@@ -64,7 +64,7 @@ class omxFIMLFitFunction : public omxFitFunction {
 	static const int ELAPSED_HISTORY_SIZE = 5;
 
 	omxFIMLFitFunction *parent;
-	int rowwiseParallel;
+  int rowwiseParallel;
 	omxMatrix* cov;				// Covariance Matrix
 	omxMatrix* means;			// Vector of means
 	omxData* data;				// The data
@@ -116,10 +116,10 @@ class omxFIMLFitFunction : public omxFitFunction {
 	int contDensityCount;
 
 	virtual ~omxFIMLFitFunction();
-	virtual void init();
-	virtual void compute(int ffcompute, FitContext *fc);
-	virtual void populateAttr(SEXP algebra);
-	virtual void invalidateCache();
+	virtual void init() override;
+	virtual void compute2(int ffcompute, FitContext *fc) override;
+	virtual void populateAttr(SEXP algebra) override;
+	virtual void invalidateCache() override;
 
 	// --- old stuff below
 
@@ -191,25 +191,25 @@ class mvnByRow {
 		std::vector<bool> &isOrdinal;
 		std::vector<bool> &isMissing;
 		bool wantOrdinal;
-	subsetOp(std::vector<bool> &_isOrdinal,
-		 std::vector<bool> &_isMissing) : isOrdinal(_isOrdinal), isMissing(_isMissing) {};
+	subsetOp(std::vector<bool> &u_isOrdinal,
+		 std::vector<bool> &u_isMissing) : isOrdinal(u_isOrdinal), isMissing(u_isMissing) {};
 		// true to include
 		bool operator()(int gx) { return !((wantOrdinal ^ isOrdinal[gx]) || isMissing[gx]); };
 	} op;
 
-	mvnByRow(FitContext *_fc, omxFitFunction *_localobj,
-		 omxFIMLFitFunction *_parent, omxFIMLFitFunction *_ofiml)
+	mvnByRow(FitContext *u_fc, omxFitFunction *u_localobj,
+		 omxFIMLFitFunction *u_parent, omxFIMLFitFunction *u_ofiml)
 	:
-	ofo((omxFIMLFitFunction*) _localobj),
+	ofo((omxFIMLFitFunction*) u_localobj),
 		shared_ofo(ofo->parent? ofo->parent : ofo),
-		expectation(_localobj->expectation),
+		expectation(u_localobj->expectation),
 		ol(ofo->ol),
 		indexVector(shared_ofo->indexVector),
 		rows(int(indexVector.size())),
 		sameAsPrevious(shared_ofo->sameAsPrevious),
 		thresholdCols(expectation->getThresholdInfo()),
 		dataColumns(expectation->getDataColumns()),
-		isOrdinal(_ofiml->isOrdinal),
+		isOrdinal(u_ofiml->isOrdinal),
 		rowMult(shared_ofo->rowMult),
 		op(isOrdinal, isMissing)
 	{
@@ -222,14 +222,14 @@ class mvnByRow {
 		firstRow = true;
 		cov = ofo->cov;
 		means = ofo->means;
-		fc = _fc;
-		ofiml = _ofiml;
-		parent = _parent;
+		fc = u_fc;
+		ofiml = u_ofiml;
+		parent = u_parent;
 		rowLikelihoods = ofiml->rowLikelihoods;
 		returnVector = ofiml->returnVector;
 		wantRowLikelihoods = ofiml->wantRowLikelihoods;
 		otherRowwiseValues = ofiml->otherRowwiseValues;
-		localobj = _localobj;
+		localobj = u_localobj;
 		omxSetMatrixElement(localobj->matrix, 0, 0, 0.0);
 		numOrdinal = ofiml->numOrdinal;
 		numContinuous = ofiml->numContinuous;
@@ -240,7 +240,7 @@ class mvnByRow {
 		useSufficientSets = ofiml->useSufficientSets;
 		verbose = ofiml->verbose;
 
-		if (parent->rowwiseParallel && fc->isClone()) {
+		if (parent->openmpUser && fc->isClone()) {
 			startTime = get_nanotime();
 		}
 
@@ -250,7 +250,7 @@ class mvnByRow {
 	};
 
 	~mvnByRow() {
-		if (parent->rowwiseParallel && fc->isClone()) {
+		if (parent->openmpUser && fc->isClone()) {
 			double el1 = get_nanotime() - startTime;
 			ofo->elapsed[shared_ofo->curElapsed] = el1;
 			if (verbose >= 3) mxLog("%s: %d--%d %.2fms",
@@ -442,26 +442,26 @@ class mvnByRow {
 
 struct condContByRow : mvnByRow {
 	typedef mvnByRow super;
-	condContByRow(FitContext *_fc, omxFitFunction *_localobj,
-		      omxFIMLFitFunction *_parent, omxFIMLFitFunction *_ofiml)
-		: super(_fc, _localobj, _parent, _ofiml) {};
+	condContByRow(FitContext *u_fc, omxFitFunction *u_localobj,
+		      omxFIMLFitFunction *u_parent, omxFIMLFitFunction *u_ofiml)
+		: super(u_fc, u_localobj, u_parent, u_ofiml) {};
 	bool eval();
 };
 
 struct oldByRow : mvnByRow {
 	typedef mvnByRow super;
-	oldByRow(FitContext *_fc, omxFitFunction *_localobj,
-		 omxFIMLFitFunction *_parent, omxFIMLFitFunction *_ofiml)
-		: super(_fc, _localobj, _parent, _ofiml) {};
+	oldByRow(FitContext *u_fc, omxFitFunction *u_localobj,
+		 omxFIMLFitFunction *u_parent, omxFIMLFitFunction *u_ofiml)
+		: super(u_fc, u_localobj, u_parent, u_ofiml) {};
 	bool eval();
 };
 
 struct condOrdByRow : mvnByRow {
 	typedef mvnByRow super;
-	condOrdByRow(FitContext *_fc, omxFitFunction *_localobj,
-		     omxFIMLFitFunction *_parent, omxFIMLFitFunction *_ofiml)
-		: super(_fc, _localobj, _parent, _ofiml) {};
+	condOrdByRow(FitContext *u_fc, omxFitFunction *u_localobj,
+		     omxFIMLFitFunction *u_parent, omxFIMLFitFunction *u_ofiml)
+		: super(u_fc, u_localobj, u_parent, u_ofiml) {};
 	bool eval();
 };
 
-#endif /* _OMXFIMLFITFUNCTION_H_ */
+#endif /* u_OMXFIMLFITFUNCTION_H_ */
