@@ -69,12 +69,15 @@ class omxMatrix {
 	int shape;
 	bool allocationLock;   // whether the data can be moved
  public:
+	double* data;						// Actual Data Pointer
+	SEXP owner;	// The R object owning data or NULL if we own it.
  	omxMatrix() : dependsOnParametersCache(false), dependsOnDefVarCache(false), joinKey(-1),
-								joinModel(0), shape(0), allocationLock(false),
+								joinModel(0), shape(0), allocationLock(false), data(0), owner(0),
 								freeRownames(false), freeColnames(false)
 		{};
 	struct dtor;
   void clearDimnames();
+  void disconnect();
 	void setDependsOnParameters() { dependsOnParametersCache = true; };
 	void setDependsOnDefinitionVariables() { dependsOnDefVarCache = true; };
 	bool dependsOnParameters() const { return dependsOnParametersCache; };
@@ -94,7 +97,6 @@ class omxMatrix {
 										//TODO: Improve encapsulation
 /* Actually Useful Members */
 	int rows, cols;						// Matrix size  (specifically, its leading edge)
-	double* data;						// Actual Data Pointer
 	unsigned short colMajor;			// used for quick transpose
 	unsigned short hasMatrixNumber;		// is this object in the matrix or algebra arrays?
 	int matrixNumber;					// the offset into the matrices or algebras arrays
@@ -103,8 +105,6 @@ class omxMatrix {
     matrixNumber = xx;
   };
 
-	SEXP owner;	// The R object owning data or NULL if we own it.
-
 /* For BLAS Multiplication Speedup */ 	// TODO: Replace some of these with inlines or macros.
 	const char* majority;				// Filled by compute(), included for speed
 	const char* minority;				// Filled by compute(), included for speed
@@ -208,7 +208,16 @@ struct omxMatrix::dtor {
 	void operator()(omxMatrix *om) { omxFreeMatrix(om); }
 };
 
-typedef std::unique_ptr< omxMatrix, omxMatrix::dtor > omxMatrixPtr;
+class omxMatrixPtr : public std::unique_ptr< omxMatrix, omxMatrix::dtor > {
+public:
+  omxMatrixPtr() = default;
+  omxMatrixPtr(omxMatrixPtr&& u) noexcept = default;
+  omxMatrixPtr(omxMatrix *a) { reset(a); }
+  omxMatrixPtr &operator=(omxMatrix *a) { reset(a); return *this; };
+  omxMatrixPtr &operator=(omxMatrixPtr &a) { reset(a.get()); a.reset(nullptr); return *this; }
+  // confuses compiler?
+  // operator omxMatrix*() const { return get(); };
+};
 
 /* Matrix Creation Functions */
 omxMatrix* omxNewMatrixFromRPrimitive0(SEXP rObject, omxState* state,
@@ -588,9 +597,31 @@ double trace_prod(const Eigen::MatrixBase<T1> &t1, const Eigen::MatrixBase<T2> &
 	return sum;
 }
 
+//For the trace of the product of two symmetric matrices:
+template <typename T1, typename T2>
+double trace_prod_symm(const Eigen::MatrixBase<T1> &t1, const Eigen::MatrixBase<T2> &t2)
+{
+	int r=0, c=0;
+	double sum = 0.0;
+	for(c=0; c < t1.rows(); c++){
+		for(r=c; r < t1.rows(); r++){
+			sum += (r==c) ? t1(r,c)*t2(r,c) : 2*t1(r,c)*t2(r,c);
+		}
+	}
+	return sum;
+}
+
 void MoorePenroseInverseSq(Eigen::Ref<Eigen::MatrixXd> mat);
 void MoorePenroseInverse(Eigen::Ref<Eigen::MatrixXd> in, Eigen::Ref<Eigen::MatrixXd> out);
 
+template <typename T1>
+void subtractFromIdentityMatrixInPlace(Eigen::MatrixBase<T1> &M)
+{
+	M *= -1.0;
+	M.diagonal().array() += 1.0;
+}
+
+
 // https://forum.kde.org/viewtopic.php?f=74&t=96706
 // https://forum.kde.org/viewtopic.php?f=74&t=124421
 // https://forum.kde.org/viewtopic.php?f=74&t=91271
