@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2019 by the individuals mentioned in the source code history
+ * Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -54,7 +54,7 @@ unsigned omxRAMExpectation::ApcIO::getVersion(FitContext *fc)
 
 template <typename T>
 void omxRAMExpectation::ApcIO::
-_refresh(FitContext *fc, T &mat, double sign)
+u_refresh(FitContext *fc, T &mat, double sign)
 {
 	omxMatrix *A = !fc? A0 : fc->state->lookupDuplicate(A0);
 	if (sign == 1) {
@@ -77,7 +77,7 @@ unsigned omxRAMExpectation::SpcIO::getVersion(FitContext *fc)
 }
 
 template <typename T>
-void omxRAMExpectation::SpcIO::_refresh(FitContext *fc, T &mat)
+void omxRAMExpectation::SpcIO::u_refresh(FitContext *fc, T &mat)
 {
 	omxMatrix *S = !fc? S0 : fc->state->lookupDuplicate(S0);
 	omxRecompute(S, fc);
@@ -96,6 +96,18 @@ void omxRAMExpectation::getExogenousPredictors(std::vector<int> &out)
 	out = exoDataColumns;
 }
 
+int omxRAMExpectation::numLatentVars() const
+{
+  if (rram) mxThrow("omxRAMExpectation::numLatentVars() not implemented for multilevel models");
+  return F->cols - F->rows;
+}
+
+int omxRAMExpectation::numObservedStats()
+{
+  if (!rram) return super::numObservedStats();
+  return rram->numObservedStats();
+}
+
 void omxRAMExpectation::compute(FitContext *fc, const char *what, const char *how)
 {
 	omxRAMExpectation* oro = this;
@@ -149,8 +161,7 @@ void omxRAMExpectation::populateAttr(SEXP robj)
 			Rf_setAttrib(expCovExt, R_DimNamesSymbol, dimnames);
 		}
 
-		ProtectedSEXP RnumStats(Rf_ScalarReal(omxDataDF(data)));
-		Rf_setAttrib(robj, Rf_install("numStats"), RnumStats);
+    super::populateAttr(robj);
 	}
 
 	MxRList out;
@@ -213,10 +224,6 @@ void omxRAMExpectation::CalculateRAMCovarianceAndMeans(FitContext *fc)
 	if (M) {
 		EigenVectorAdaptor Emean(means);
 		pcalc.mean(fc, Emean);
-		if (slope) {
-			EigenMatrixAdaptor Eslope(slope);
-			Emean += Eslope * exoPredMean;
-		}
 		if(OMX_DEBUG_ALGEBRA) {omxPrintMatrix(means, "....RAM: Model-implied Means Vector:");}
 	}
 }
@@ -240,7 +247,7 @@ static void recordNonzeroCoeff(omxMatrix *m, std::vector<coeffLoc> &vec, bool lo
 }
 
 omxRAMExpectation::omxRAMExpectation(omxState *st, int num)
-	: super(st, num), studiedF(false), numExoPred(0), slope(0), rram(0)
+	: super(st, num), studiedF(false), openBox(false), numExoPred(0), slope(0), rram(0)
 {
 	if (st->isClone()) {
 		omxRAMExpectation *ram = (omxRAMExpectation *)st->getParent(this);
@@ -379,15 +386,32 @@ void omxRAMExpectation::init()
 	if(OMX_DEBUG) { mxLog("Generating internals for computation."); }
 
 	cov = omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedCovariance", l, l);
-	if (!cov->hasMatrixNumber) covOwner = omxMatrixPtr(cov);
-	else connectMatrixToExpectation(cov, this, "covariance");
+	if (!cov->hasMatrixNumber) covOwner = cov;
+	else {
+    connectMatrixToExpectation(cov, this, "covariance");
+    openBox = true;
+  }
+  fullCov = omxNewMatrixFromSlot(rObj, currentState, "expectedFullCovariance");
+  if (fullCov) {
+    connectMatrixToExpectation(fullCov, this, "fullCov");
+    openBox = true;
+  }
 
 	if (M) {
 		means =	omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedMean", 1, l);
-		if (!means->hasMatrixNumber) meanOwner = omxMatrixPtr(means);
-		else connectMatrixToExpectation(means, this, "mean");
+		if (!means->hasMatrixNumber) meanOwner = means;
+		else {
+      connectMatrixToExpectation(means, this, "mean");
+      openBox = true;
+    }
+    fullMean = omxNewMatrixFromSlot(rObj, currentState, "expectedFullMean");
+    if (fullMean) {
+      connectMatrixToExpectation(fullMean, this, "fullMean");
+      openBox = true;
+    }
 	} else {
 		RAMexp->means  = 	NULL;
+    fullMean = 0;
 	}
 
 	RAMexp->studyF();
@@ -413,10 +437,18 @@ void omxRAMExpectation::init()
 		sio->S0 = S;
 
 		pcalc.attach(k, l, latentFilter, isProductNode, mio, aio, sio);
+
+    omxMatrix *selVec = omxNewMatrixFromSlot(rObj, currentState, "selectionVector");
+    if (selVec) {
+      DataFrame selPlan(rObj.slot("selectionPlan"));
+      pcalc.attachSelection(selVec, selPlan);
+    }
+    pcalc.attachFullMemory(fullMean, fullCov);
 		pcalc.setAlgo(0, hasProductNodes, useSparse);
 
 		currentState->restoreParam(estSave);
-	} else {
+	}
+  if (!currentState->isTopState()) {
     auto pex = (omxRAMExpectation*) currentState->getParent(this);
     if (pex->slope) {
       numExoPred = pex->numExoPred;
@@ -428,7 +460,34 @@ void omxRAMExpectation::init()
 
 void omxRAMExpectation::studyExoPred()
 {
-	if (data->defVars.size() == 0 || !M || !M->isSimple() || !S->isSimple()) return;
+  if (!currentState->isTopState()) return; // done in init(); defVar already modified
+  // We check each condition separately so we can log the outcome, if requested.
+	if (data->defVars.size() == 0) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: no def vars", name);
+    return;
+  }
+  if (verbose >= 1) mxLog("%s::studyExoPred: found %d def vars",
+                          name, int(data->defVars.size()));
+  if (!M) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: no means", name);
+    return;
+  }
+  if (M->isAlgebra()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: means model is an algebra", name);
+    return;
+  }
+  if (M->populateDependsOnDefinitionVariables()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: means model depends on def vars", name);
+    return;
+  }
+  if (S->isAlgebra()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: S (covariance) is an algebra", name);
+    return;
+  }
+  if (S->populateDependsOnDefinitionVariables()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: S (covariance) depends on def vars", name);
+    return;
+  }
 
 	Eigen::VectorXd estSave;
 	currentState->setFakeParam(estSave);
@@ -450,7 +509,11 @@ void omxRAMExpectation::studyExoPred()
 				if (latentFilter[cx]) toManifest = true;
 				else latentName = S->colnames[cx];
 			}
-			if (!toManifest && !latentName) continue;
+			if (!toManifest && !latentName) {
+        if (verbose >= 1) mxLog("%s::studyExoPred: def var '%s' has unknown effect",
+                                name, data->columnName(dv.column));
+        continue;
+      }
 			if (latentName) mxThrow("%s: latent exogenous variables are not supported (%s -> %s)", name,
 						 S->colnames[dv.col], latentName);
 			exoDataColIndex[dv.col] = dv.column;
@@ -495,18 +558,6 @@ void omxRAMExpectation::addSlopeMatrix()
 	}
 }
 
-void omxRAMExpectation::connectToData()
-{
-  super::connectToData();
-
-	exoPredMean.resize(exoDataColumns.size());
-	for (int cx=0; cx < int(exoDataColumns.size()); ++cx) {
-		auto &e1 = data->rawCol( exoDataColumns[cx] );
-		Eigen::Map< Eigen::VectorXd > vec(e1.d(), data->numRawRows());
-		exoPredMean[cx] = vec.mean();
-	}
-}
-
 void omxRAMExpectation::studyF()
 {
 	// Permute the data columns such that the manifest
@@ -623,12 +674,12 @@ void omxRAMExpectation::analyzeDefVars(FitContext *fc)
 
 		if (M && dv.matrix == mNum) {
 			dvInfluenceMean[k] = true;
-			dvInfluenceVar[k] = dvInfluenceVar[k] | (hasVariance[ dv.col ] != 0.0);
+			dvInfluenceVar[k] = dvInfluenceVar[k] || (hasVariance[ dv.col ] != 0.0);
 			dvContribution[dv.col].insert(std::make_pair(this, k));
 			continue;
 		}
 		if (dv.matrix == sNum) {
-			dvInfluenceMean[k] = dvInfluenceMean[k] | (hasMean[ dv.col ] != 0.0);
+			dvInfluenceMean[k] = dvInfluenceMean[k] || (hasMean[ dv.col ] != 0.0);
 			dvInfluenceVar[k] = true;
 			dvContribution[dv.col].insert(std::make_pair(this, k));
 			continue;
@@ -806,7 +857,7 @@ namespace RelationalRAMExpectation {
 	}
 
 	template <typename T>
-	void independentGroup::ApcIO::_refresh(FitContext *fc, T &mat, double sign)
+	void independentGroup::ApcIO::u_refresh(FitContext *fc, T &mat, double sign)
 	{
 		for (int ax=0; ax < clumpSize; ++ax) {
 			placement &pl = par.placements[ax];
@@ -877,7 +928,7 @@ namespace RelationalRAMExpectation {
 	}
 
 	template <typename T>
-	void independentGroup::SpcIO::_refresh(FitContext *fc, T &mat)
+	void independentGroup::SpcIO::u_refresh(FitContext *fc, T &mat)
 	{
 		for (int ax=0; ax < clumpSize; ++ax) {
 			placement &pl = par.placements[ax];
@@ -912,8 +963,8 @@ namespace RelationalRAMExpectation {
 			rowToLayoutMap[ std::make_pair(data, frow) ] = -1;
 		}
 
-		struct addr a1;
-		struct addrSetup as1;
+		class addr a1;
+		class addrSetup as1;
 		as1.clumped = false;
 		a1.ig = 0;
 		as1.parent1 = NA_INTEGER;
@@ -1001,7 +1052,7 @@ namespace RelationalRAMExpectation {
 
 	struct CompareLib {
 		state &st;
-		CompareLib(state *_st) : st(_st->getParent()) {};
+		CompareLib(state *u_st) : st(u_st->getParent()) {};
 
 		// actually stores !missingness
 		template <typename T>
@@ -1109,7 +1160,7 @@ namespace RelationalRAMExpectation {
 	};
 
 	struct CompatibleCovCompare : CompareLib {
-		CompatibleCovCompare(state *_st) : CompareLib(_st) {};
+		CompatibleCovCompare(state *u_st) : CompareLib(u_st) {};
 
 		bool operator() (const std::vector<int> &lhs, const std::vector<int> &rhs) const
 		{
@@ -1126,7 +1177,7 @@ namespace RelationalRAMExpectation {
 	};
 
 	struct CompatibleMeanCompare : CompareLib {
-		CompatibleMeanCompare(state *_st) : CompareLib(_st) {};
+		CompatibleMeanCompare(state *u_st) : CompareLib(u_st) {};
 
 		addr *joinedWith(const addr &la, int jx) const
 		{
@@ -1180,7 +1231,7 @@ namespace RelationalRAMExpectation {
 	};
 
 	struct RampartTodoCompare : CompareLib {
-		RampartTodoCompare(state *_st) : CompareLib(_st) {};
+		RampartTodoCompare(state *u_st) : CompareLib(u_st) {};
 
 		bool operator() (const addr *lhs, const addr *rhs) const
 		{
@@ -1195,7 +1246,7 @@ namespace RelationalRAMExpectation {
 	};
 
 	struct RampartClumpCompare : CompareLib {
-		RampartClumpCompare(state *_st) : CompareLib(_st) {};
+		RampartClumpCompare(state *u_st) : CompareLib(u_st) {};
 
 		bool clumpCmp(const int lhs, const int rhs) const {
 			bool mismatch;
@@ -1509,8 +1560,8 @@ namespace RelationalRAMExpectation {
 		gMap.push_back(ax);
 	}
 
-	independentGroup::independentGroup(class state *_st, int size, int _clumpSize)
-		: st(*_st), clumpSize(_clumpSize)
+	independentGroup::independentGroup(class state *u_st, int size, int u_clumpSize)
+		: st(*u_st), clumpSize(u_clumpSize)
 	{
 		placements.reserve(size);
 	}
@@ -1729,7 +1780,7 @@ namespace RelationalRAMExpectation {
 				std::vector<int> dc(a1.clump);
 				std::sort(dc.begin(), dc.end(), RampartClumpCompare(this));
 				for (size_t cx=0; cx < dc.size(); ++cx) {
-					if (dc[cx] != a1.clump[cx]) mxThrow("oops");
+					if (dc[cx] != a1.clump[cx]) OOPS;
 				}
 			}
 		}
@@ -1739,7 +1790,7 @@ namespace RelationalRAMExpectation {
 	template <bool model>
 	struct UnitAccessor {
 		state &st;
-		UnitAccessor(state *_st) : st(*_st) {};
+		UnitAccessor(state *u_st) : st(*u_st) {};
 		bool isModel() const { return model; };
 
 		// split into coeff & coeffRef versions TODO
@@ -1861,6 +1912,19 @@ namespace RelationalRAMExpectation {
 		}
 	}
 
+	int state::numObservedStats()
+  {
+    int stats = 0;
+		for (auto *ex : allEx) {
+			omxRAMExpectation *ram1 = (omxRAMExpectation*) ex;
+      auto &dc = ram1->getDataColumns();
+      stats += dc.size() * ram1->data->nrows();
+
+      // no thresholds since our multilevel is limited to continuous only
+    }
+    return stats;
+  }
+
 	void state::init(omxExpectation *expectation, FitContext *fc)
 	{
 		parent = this;
@@ -1898,6 +1962,17 @@ namespace RelationalRAMExpectation {
 		hasProductNodes = false;
 		for (auto *ex : allEx) {
 			omxRAMExpectation *ram1 = (omxRAMExpectation*) ex;
+      if (ram1->isOpenBox()) {
+        if (ex != homeEx) {
+          mxThrow("Expectation '%s' is part of a multilevel model with '%s';"
+                  "introspection of per-level expectations is not possible",
+                  ex->name, homeEx->name);
+        } else {
+          mxThrow("Expectation '%s' is part of a multilevel model;"
+                  "introspection of per-level expectations is not possible",
+                  ex->name);
+        }
+      }
 			hasProductNodes |= ram1->getHasProductNodes();
 			if (!ex->data->hasWeight() && !ex->data->hasFreq()) continue;
 			mxThrow("%s: row frequencies or weights provided in '%s' are not compatible with joins",
@@ -2079,7 +2154,7 @@ namespace RelationalRAMExpectation {
 
 	struct SimUnitAccessor {
 		state &st;
-		SimUnitAccessor(state *_st) : st(*_st) {};
+		SimUnitAccessor(state *u_st) : st(*u_st) {};
 
 		double &operator() (const int unit, const int obs)
 		{
@@ -2323,8 +2398,11 @@ namespace RelationalRAMExpectation {
 			//Rf_setAttrib(dv, R_NamesSymbol, obsNameVec);
 			dbg.add("dataVec", dv);
 		} else {
-			Rf_warning("%s: group %d too large to transfer to back to R",
-				   st.homeEx->name, arrayIndex+1);
+      if (!Global->RAMmultilevelWarning) {
+        Rf_warning("%s: group %d too large to transfer to back to R",
+                   st.homeEx->name, arrayIndex+1);
+        Global->RAMmultilevelWarning = true;
+      }
 		}
 
 		SEXP aIndex, modelStart, obsStart;
@@ -2353,18 +2431,20 @@ namespace RelationalRAMExpectation {
 		dbg.add("numSufficientSets", Rcpp::wrap(int(sufficientSets.size())));
 		dbg.add("fit", Rcpp::wrap(fit));
 
-		int digits = ceilf(log10f(sufficientSets.size()));
-		std::string fmt = string_snprintf("ss%%0%dd", digits);
-		for (size_t gx=0; gx < sufficientSets.size(); ++gx) {
-			sufficientSet &ss = sufficientSets[gx];
-			MxRList info;
-			info.add("start", Rcpp::wrap(1 + ss.start));
-			info.add("length", Rcpp::wrap(ss.length));
-			info.add("mean", Rcpp::wrap(ss.dataMean));
-			info.add("covariance", Rcpp::wrap(ss.dataCov));
-			std::string name = string_snprintf(fmt.c_str(), int(1+gx));
-			dbg.add(name.c_str(), info.asR());
-		}
+    if (sufficientSets.size()) {
+      int digits = ceilf(log10f(sufficientSets.size()));
+      std::string fmt = string_snprintf("ss%%0%dd", digits);
+      for (size_t gx=0; gx < sufficientSets.size(); ++gx) {
+        sufficientSet &ss = sufficientSets[gx];
+        MxRList info;
+        info.add("start", Rcpp::wrap(1 + ss.start));
+        info.add("length", Rcpp::wrap(ss.length));
+        info.add("mean", Rcpp::wrap(ss.dataMean));
+        info.add("covariance", Rcpp::wrap(ss.dataCov));
+        std::string name = string_snprintf(fmt.c_str(), int(1+gx));
+        dbg.add(name.c_str(), info.asR());
+      }
+    }
 	}
 
 	void state::exportInternalState(MxRList &dbg)
