@@ -4,13 +4,13 @@ context("loadDataByRow")
 suppressWarnings(RNGversion("3.5"))
 set.seed(1)
 
-if (mxOption(NULL,"Default optimizer") == 'NPSOL') stop("SKIP")
+skip_if(mxOption(key="Default optimizer") == 'NPSOL')
 #mxOption(NULL, "Number of Threads", 1L)
 
 data("jointdata", package ="OpenMx")
 
 # specify ordinal columns as ordered factors
-jointdata[,c(2,4,5)] <- mxFactor(jointdata[,c(2,4,5)], 
+jointdata[,c(2,4,5)] <- mxFactor(jointdata[,c(2,4,5)],
 	levels=list(c(0,1), c(0, 1, 2, 3), c(0, 1, 2)))
 
 satCov <- mxMatrix("Symm", 5, 5,
@@ -25,10 +25,10 @@ loadings$ubound[1,4:5] <- 2
 
 resid <- mxMatrix("Diag", 5, 5,
 	free=c(TRUE, FALSE, TRUE, FALSE, FALSE), values=.5, name="U")
-	
+
 means <- mxMatrix("Full", 1, 5,
 	free=c(TRUE, FALSE, TRUE, FALSE, FALSE), values=0, name="M")
-	
+
 thresh <- mxMatrix("Full", 3, 3, FALSE, 0, name="T")
 
 thresh$free[,1] <- c(TRUE, FALSE, FALSE)
@@ -95,12 +95,11 @@ model3$compute$steps[['LD']]$method <- 'csv'
 model3Fit <- mxRun(model3)
 
 omxCheckEquals(model3Fit$compute$steps[['LD']]$debug$loadCounter, 1L)
+expect_equal(model3Fit$data$observedStats[['numEstimatedEntries']], 57)
 
 discardCols <- c("OpenMxEvals", "iterations", "timestamp",
                  "MxComputeLoop1", "objective", "statusCode", "fitUnits",
                  'testCols.csv:z1', 'testCols.csv:z2')
-thr <- c(7, 8, 7, 7, 8, 8, 7, 8, 8, 7, 7, 8, 7, 7, 4, 8, 8, 8, 9, 8,
-         9, 9, 11, 10, 10, 9, 9, 8, 10, 10) - 2
 
 log <- model3Fit$compute$steps[['CPT']]$log
 
@@ -111,14 +110,14 @@ for (col in discardCols) log[[col]] <- NULL
 lmad <- -log10(apply(abs(as.matrix(log - result1)), 2, max))
 # names(lmad) <- c()
 # cat(deparse(floor(lmad)))
-# print(lmad - thr)
-omxCheckTrue(all(lmad - thr > 0))
+# print(lmad - 7)
+omxCheckTrue(all(lmad - 7 > 0))
 
 # --------------
 
 totalCol <- ncol(flat) + length(letters)
 lcol <- sample.int(totalCol, length(letters))
-print(sort(lcol))
+#print(sort(lcol))
 flatMap <- (1:totalCol)[-lcol]
 flat2 <- matrix("", nrow(flat), totalCol)
 flat2[,lcol] <- letters
@@ -136,5 +135,5 @@ for (col in discardCols) log[[col]] <- NULL
 lmad <- -log10(apply(abs(as.matrix(log - result1)), 2, max))
 # names(lmad) <- c()
 # cat(deparse(floor(lmad)))
-# print(lmad - thr)
-omxCheckTrue(all(lmad - thr > 0))
+# print(lmad - 7)
+omxCheckTrue(all(lmad - 7 > 0))
